package backend

import (
	"fmt"
	"log"
)

func (db *DB) WriteArticleTags(articleID int64, tagIDs []int64) error {
	query := "INSERT INTO articles_tags (article_id, tag_id) VALUES (?, ?)"

	for i := 0; i < TxMaxRetries; i++ {
		err := func() error {
			tx, err := db.Begin()
			if err != nil {
				return fmt.Errorf("error starting transaction: %v", err)
			}

			for _, tagID := range tagIDs {
				if _, err := tx.Exec(query, articleID, tagID); err != nil {
					if rollbackErr := tx.Rollback(); rollbackErr != nil {
						log.Fatalf("transaction error: %v, rollback error: %v", err, rollbackErr)
					}
					return fmt.Errorf("error inserting into articles_tags: %v", err)
				}
			}

			if err = tx.Commit(); err != nil {
				return fmt.Errorf("error committing transaction: %v", err)
			}
			return nil
		}()
		if err == nil {
			return nil
		}

		log.Println(err)
		wait(i)
	}
	return fmt.Errorf("error: %v unsuccessful retries for DB operation, aborting", TxMaxRetries)
}

func (db *DB) GetArticleTags(articleID int64) ([]*Tag, error) {
	query := `
    SELECT t.id, t.name
    FROM articles a
    INNER JOIN articles_tags at ON a.id = at.article_id
    INNER JOIN tags t ON at.tag_id = t.id
    WHERE a.id = ?
    `
	rows, err := db.Query(query, articleID)
	if err != nil {
		return nil, fmt.Errorf("error querying articles_tags: %v", err)
	}

	tags := make([]*Tag, 0)
	for rows.Next() {
		tag := new(Tag)
		if err = rows.Scan(&tag.ID, &tag.Name); err != nil {
			return nil, fmt.Errorf("error scanning rows: %v", err)
		}
		tags = append(tags, tag)
	}

	return tags, nil
}

func (db *DB) UpdateArticleTags(articleID int64, tagIDs []int64) error {
	deleteQuery := "DELETE FROM articles_tags WHERE article_id = ?"
	insertQuery := "INSERT INTO articles_tags (article_id, tag_id) VALUES (?, ?)"

	for i := 0; i < TxMaxRetries; i++ {
		err := func() error {
			tx, err := db.Begin()
			if err != nil {
				return fmt.Errorf("error starting transaction: %v", err)
			}

			if _, err := tx.Exec(deleteQuery, articleID); err != nil {
				if rollbackErr := tx.Rollback(); rollbackErr != nil {
					log.Fatalf("transaction error: %v, rollback error: %v", err, rollbackErr)
				}
				return fmt.Errorf("error deleting entries from articles_tags before inserting new ones: %v", err)
			}

			for _, tagID := range tagIDs {
				if _, err := tx.Exec(insertQuery, articleID, tagID); err != nil {
					if rollbackErr := tx.Rollback(); rollbackErr != nil {
						log.Fatalf("transaction error: %v, rollback error: %v", err, rollbackErr)
					}
					return fmt.Errorf("error inserting new entries into articles_tags: %v", err)
				}
			}

			if err = tx.Commit(); err != nil {
				return fmt.Errorf("error committing transaction: %v", err)
			}
			return nil
		}()
		if err == nil {
			return nil
		}

		log.Println(err)
		wait(i)
	}
	return fmt.Errorf("error: %v unsuccessful retries for DB operation, aborting", TxMaxRetries)
}
